import DiscreteRandomVariable.DiscreteRandomVariable as DiscreteRandomVariable

class OutputDomain(DiscreteRandomVariable.DiscreteRandomVariable):

    # Input: The probability mass function of the input domain, InputDomain_ProbabilityMassFunction. The probability mass function of the output
    #        domain can be determined from this.
    #        Function: This is a dictionary, such that each key in the dictionary is an input from the input domain, and the associated value
    #        is the output that this input leads to
    # Process: We first compute the alphabet of the output domain, which is the set of distinct values in the output domain
    #          We then compute all of the inverse images, based on the input and output domain
    #          We finally compute the probability mass function, based on the probability mass function of the input domain
    def __init__(self, InputDomain_ProbabilityMassFunction, Function):
        self.Alphabet = list(set(Function.values()))
        self.InverseImages = self.compute_Inverse_Image_Of_Each_Output(Function)
        self.ProbabilityMassFunction = self.compute_Probability_Mass_Function(InputDomain_ProbabilityMassFunction)

    # Input: A Function is a dictionary, such that each key is an input, and the corresponding value is the output for that input.
    # Process: This method computes the Inverse Image for each Output
    # Output: returns the Inverse Images as output. Inverse Images is a dictionary, such that the input is 
    def compute_Inverse_Image_Of_Each_Output(self,Function):
        InverseImages = {}
        
        for Input in list(Function.keys()):
            Output = Function[Input]
            if Output not in list(InverseImages.keys()):
                InverseImages[Output] = [Input]
            else:
                InverseImage = InverseImages[Output]
                InverseImage.append(Input)
                InverseImages[Output] = InverseImage
        return InverseImages
    
    # Prints out the inverse images of each output
    def print_InverseImages(self):
        for Output in list(self.InverseImages.keys()):
            print("Output: " + str(Output))
            for Input in self.InverseImages[Output]:
                print("\t"+str(Input))
            print()
    
    # Input: InputDomain_ProbabilityMassFunction, which is the probability mass function of the input domain
    # Process: Computed the probability mass function of the output domain, based on the probability mass function of the input domain
    # Output: The probability mass function of the output domain
    def compute_Probability_Mass_Function(self,InputDomain_ProbabilityMassFunction):
        ProbabilityMassFunction = {}
        for Output in self.Alphabet:
            InverseImage = self.InverseImages[Output]
            Probability = 0
            for Input in InverseImage:
                Probability = Probability + InputDomain_ProbabilityMassFunction[Input]
            ProbabilityMassFunction[Output] = Probability
        return ProbabilityMassFunction


    # This method computes the size of the input domain, based on the inverse images.
    def compute_InputDomainSize(self):
        InputDomainSize = 0
        for Output in list(self.InverseImages.keys()):
            InverseImage = self.InverseImages[Output]
            InputDomainSize = InputDomainSize + len(InverseImage)
        return InputDomainSize
    
    
    
    
